"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.renderResultSummary = void 0;
const log_1 = require("../../log");
const config_1 = require("../../config");
const types_1 = require("../types");
const traverse_1 = require("../tools/traverse");
const record_1 = require("./record");
const predicates_1 = require("./predicates");
const { Unit, Group } = types_1.CommandType;
const { Collapsed, Hidden } = types_1.DisplayState;
const getResultType = (history) => {
  const last = history[history.length - 1];
  if (!last) return "nd"; // No data. Shouldn't happen
  switch (last.event) {
    case types_1.Event.Start:
      return "nc"; // Not completed. Shouldn't happen
    case types_1.Event.Success:
      return "ok";
    case types_1.Event.Failure:
      return "err";
  }
};
const resultTypeWidth = 4;
const durationWidth = 5;
const durationFractionDigits = 2;
const resultWidth = resultTypeWidth + durationWidth + 2;
const printResultField = (command, recorder) => {
  const history = recorder.getHistory(command);
  const resultType = getResultType(history);
  const duration = record_1.calcDurationSec(history);
  const { stdout } = process;
  stdout.write(resultType.padEnd(resultTypeWidth));
  const durationStr = String(duration.toFixed(durationFractionDigits));
  stdout.write(durationStr.padStart(durationWidth));
  stdout.write("s ");
};
const renderResultField = (command, recorder) => {
  if (predicates_1.shouldCalc(command)) printResultField(command, recorder);
  else process.stdout.write(" ".repeat(resultWidth));
};

/** Outputs result summary in a list form. */
const renderList = (topCommand, recorder) => {
  const { stdout } = process;
  traverse_1.depthFirstSearch(topCommand, (command) => {
    if (command.displayState === Hidden) return false;
    if (command.type === Group && command.displayState !== Collapsed)
      return false;
    printResultField(command, recorder);
    stdout.write("| ");
    stdout.write(getDisplayName(command));
    stdout.write("\n");
    return false;
  });
};
const getDisplayName = (command) => {
  let prefix;
  switch (command.type) {
    case Unit:
      prefix = "";
      break;
    case Group:
      switch (command.subType) {
        case types_1.CommandSubType.Sequence:
          prefix = "[seq] ";
          break;
        case types_1.CommandSubType.Parallel:
          prefix = "[par] ";
          break;
        default:
          prefix = "[?] ";
          log_1.warn(
            `Command type mismatch: (${command.type}, ${command.subType}`
          );
          break;
      }
  }
  return prefix + command.name;
};

/** Outputs result summary in a tree form. */
const renderTree = (topCommand, recorder) => {
  const { stdout } = process;
  traverse_1.depthFirstSearch(topCommand, (command, depth) => {
    if (command.displayState === Hidden) return false;
    renderResultField(command, recorder);
    stdout.write("| ");
    stdout.write("  ".repeat(depth));
    stdout.write(getDisplayName(command));
    stdout.write("\n");
    return false;
  });
};

/** Writes result summary into standard output. */
const renderResultSummary = (topCommand, recorder) => {
  switch (config_1.resultSummaryType) {
    case config_1.ResultSummaryType.Tree:
      renderTree(topCommand, recorder);
      break;
    case config_1.ResultSummaryType.List:
      renderList(topCommand, recorder);
      break;
    case null:
      break;
    case undefined:
      break;
    default:
      log_1.warn(`Unknown result summary type: ${config_1.resultSummaryType}`);
  }
};
exports.renderResultSummary = renderResultSummary;
